#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>
#include <QAction>
#include <QStatusBar>
#include <QMessageBox>
#include <QApplication>
#include <QClipboard>
#include <QDomDocument>
#include "qfraktalview.h"

class QSpinBox;

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

public slots:
    /**
     * Activating this slot signals changed view content.
     */
    void slotViewContentChanged() { hasChanged=true; }

protected: // protected methods
    /**
     * Sets up the GUI actions.
     */
    void initActions();

    void initToolBar();

    /**
     * @brief Reimplementation of QWidget::closeEvent(QCloseEvent *)
     */
    void closeEvent(QCloseEvent *event);

    /**
     * @brief For getting an unique element from a saved file DOM tree.
     * @param doc DOM document of a loaded XML file
     * @param name Name of the element to get.
     * @return Element entity
     * @throw QString message when the lookup failed
     */
    QDomElement lookupElement(QDomDocument &doc,const QString &name) throw(QString)
    {
        QDomNodeList list = doc.elementsByTagName(name);
        if(list.size() != 1) throw tr("Invalid document, not one %1 element!").arg(name);
        QDomElement e = list.item(0).toElement();
        if(e.isNull()) throw tr("Invalid %1 element!").arg(name);
        return e;
    }

protected slots:
    void slotFileNew() { MainWindow *w = new MainWindow(this); w->show(); }

    void slotFileQuit() { this->close(); }

    void slotFileSave();

    void slotFileSaveAs();

    void slotFileOpen();

    void slotFileSnapshot();

    void slotEditCopy()
    {
        QApplication::clipboard()->setImage(view->getRenderedImage());
        statusbar->showMessage(tr("Image copied to clipboard."));
    }

    void slotHelpHelp() { QMessageBox::information(this,tr("QFraktal help"),helptext); }

    void slotHelpAbout() { QMessageBox::about(this,"QFraktal",about); }

    void slotHelpAboutQt() { QMessageBox::aboutQt(this,"QFraktal"); }

protected: // protected attributes
    QAction *fileNew; /**< for creating a new instance of the program */
    QAction *fileOpen; /**< for opening a saved view */
    QAction *fileSave; /**< for saving the current view */
    QAction *fileSaveAs; /**< for saving the current view under a new name */
    QAction *fileSnapshot; /**< for saving the current image */
    QAction *fileQuit; /**< for quitting the application */
    QAction *editCopy; /**< for copying the actual image to the clipboard */
    QAction *helpHelp; /**< program description */
    QAction *helpAbout; /**< about the author */
    QAction *helpAboutQt; /**< about the Qt library */
    QString helptext; /**< short program help */
    static const QString about; /**< about the author */
    bool hasChanged; /**< flag indicating unsaved content */
    QStatusBar *statusbar; /**< status bar of the program */
    QFraktalView *view; /**< the main view to render & display the mandelbrot fractal */
    QSpinBox *iteration_spinner; /**< spinner to select the iteration depth */
    QString lastFileName; /**< last filename used to save the file */
    bool inSaveAs; /**< flag indicating that save as and not save is being processed */
    static const QString doctype; /**< doctype of the XML files */ 
};

#endif // MAINWINDOW_H
