#ifndef RENDERTHREAD_H
#define RENDERTHREAD_H

#include <QThread>
#include <QImage>
#include <QColor>
#include <QRect>
#include <QMutex>
#include <QWaitCondition>
#include <vector>
#include <complex>

using namespace std;

/**
 * @brief Rendering thread to render the fractal.
 */
class RenderThread : public QThread
{
    Q_OBJECT
public:
    /**
     * @brief constructor.
     * @param parent parent object
     * @param target Target image to render on.
     */
    RenderThread()
    {
        restart=abort=false;
    }

    /**
     * Destructor.
     */
    ~RenderThread();

    /**
     * @brief Start the rendering.
     * @param ul Upper-left of the rendered area in the complex plane.
     * @param lr Lower-right of the rendered area in the complex plane.
     * @param dr step size for the real part
     * @param di step size for the imaginary part
     * @param colormap Colormap used to map to colors, size gives the iteration depth!
     * @param region Region of the image to render
     * @param antialias Toggles anti-alias calculations on or off.
     */
    void render(complex<double> ul, complex<double> lr,double dr,double di,const vector<QRgb> colormap,QRect region,bool antialias=true);

signals:
    /**
     * @brief Emitted when a rendering has been completed.
     * @param tile rendered image tile
     * @param region region of the image tile
     */
    void renderingCompleted(const QImage &tile,const QRect &region);

protected: // protected methods
    /**
     * @brief Worker method.
     * Here the calculations take place.
     */
    void run();

protected: // protected attributes
    vector<QRgb> colormap; /**< colormap to map iteration depth -> color */
    complex<double> ul; /**< Upper-left of the rendered area in the complex plane. */
    complex<double> lr; /**< Lower-right of the rendered area in the complex plane. */
    double dr; /**< step size for the real part */
    double di; /**< step size for the imaginary part */
    QRect region; /**< Region of the image to render */
    bool antialias; /**< antialias flag */
    bool restart; /**< Flag indicating that a restart should take place. */
    bool abort; /**< Flag indicating that the thread execution should be aborted. */
    QMutex mutex;/**< for regulating attribute access */
    QWaitCondition condition; /**< for letting the thread idle wait */
};

#endif // RENDERTHREAD_H
